package net.gimite.androidirc;

import java.io.IOException;

import org.schwering.irc.lib.IRCConnection;
import org.schwering.irc.lib.IRCEventListener;
import org.schwering.irc.lib.IRCModeParser;
import org.schwering.irc.lib.IRCUser;

import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnKeyListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ScrollView;
import android.widget.TextView;

public class ChatActivity extends Activity {
    
	private ScrollView scrollBox;
	private TextView log;
	private EditText inputBox;
	private Button sayButton;
	
	private IRCConnection conn;
	private Handler handler = new Handler();
	private SharedPreferences config;
	private String host;
	private int port;
	private String myNick;
	private String channel;
	private String encoding;
	
	private static final int CONFIG_REQUEST = 0;
	
	// Called when the activity is first created.
    @Override
    public void onCreate(Bundle icicle) {
        super.onCreate(icicle);
        setContentView(R.layout.chat_activity);
        scrollBox = (ScrollView)findViewById(R.id.scrollBox);
        log = (TextView)findViewById(R.id.log);
        inputBox = (EditText)findViewById(R.id.inputBox);
        sayButton = (Button)findViewById(R.id.sayButton);
        inputBox.setKeyListener(onInputBoxKey);
        sayButton.setOnClickListener(onSayButtonClick);
        
		config = getSharedPreferences("connection", 0);
    }
    
    // Called when ConfigActibity has been closed.
	@Override
	protected void onActivityResult(int requestCode, int resultCode,
			String data, Bundle extras) {
		super.onActivityResult(requestCode, resultCode, data, extras);
		if (requestCode == CONFIG_REQUEST) {
			if (resultCode == RESULT_OK){ // User clicked [Connect].
				connect();
			}
		}
	}
    
    private void addMessage(String message) {
    	// This method do two things:
    	//   1. Append message to log widget
    	//   2. Scroll to the bottom of log
    	// Each of these steps are executed via Handler, because:
    	//   1. This method may be called from sub thread
    	//      (Accessing widgets' method directly from sub thread
    	//       is forbidden)
    	//   2. Value of log.getHeight() isn't updated if we call
    	//      it directly
    	final String constMessage = message;
    	final int oldY = log.getHeight() - scrollBox.getHeight();
    	final Runnable scroller = new Runnable(){
			public void run() {
				// Scrolls down to the bottom unless the user has
				// scrolled up beforehand.
	            if (scrollBox.getScrollY() >= oldY) {
					scrollBox.smoothScrollTo(0,
		            		log.getHeight() - scrollBox.getHeight());
	            }
			}
    	};
    	final Runnable adder = new Runnable(){
			public void run() {
				if (log.getText().length() > 0){
					log.append("\n");
				}
		    	log.append(constMessage);
		    	handler.post(scroller);
			}
    	};
    	handler.post(adder);
    }
    
    private void addMessage(String nick, String message) {
    	addMessage("<" + nick + "> " + message);
    }
    
    private void sendMessage() {
		String message = inputBox.getText().toString();
		conn.doPrivmsg(channel, message);
		addMessage(myNick, message);
		inputBox.setText("");
    }
    
    private boolean isConnected() {
    	return conn != null && conn.isConnected();
    }
    
    private void connect() {
		host = config.getString("host", "");
		port = config.getInt("port", 0);
		channel = config.getString("channel", "");
		myNick = config.getString("nick", "");
		encoding = config.getString("encoding", "");
		setTitle(channel + "@" + host);
    	if (conn != null) conn.close();
		conn = new IRCConnection(
    			host, port, port,
				null, myNick, "Android IRC", "Android IRC");
    	conn.setEncoding(encoding);
		conn.addIRCEventListener(new IRCListener());
		conn.setDaemon(true);
		conn.setColors(false);
		conn.setPong(true);
		try {
			addMessage("Connecting...");
			conn.connect();
		} catch (IOException ex) {
			addMessage("Error: " + ex.getMessage()); 
			ex.printStackTrace();
		}
    }
    
    private void updateSayButton() {
    	// Uses Handler because sub thread may call this.
    	handler.post(new Runnable(){
			public void run() {
				sayButton.setText(isConnected() ? "Say" : "Connect");
			}
    	});
    }
    
    private OnClickListener onSayButtonClick = new OnClickListener() {
        public void onClick(View v) {
        	if (isConnected()) { // [Say] button
        		sendMessage();
        	} else { // [Connect] button
        		// Opens ConfigActivity.
                Intent i = new Intent(
                		ChatActivity.this, ConfigActivity.class);
                startSubActivity(i, CONFIG_REQUEST);
        	}
        }
    };
    
    private OnKeyListener onInputBoxKey = new OnKeyListener() {
		public boolean onKey(View v, int keyCode, KeyEvent event) {
			if (event.isDown()) {
				if (keyCode == KeyEvent.KEYCODE_NEWLINE) {
					if (isConnected()) sendMessage();
					return true;
				}
			}
			return false;
		}
    };
    
    private class IRCListener implements IRCEventListener{

		public void onDisconnected() {
			addMessage("Disconnected");
			updateSayButton();
		}

		public void onError(String message) {
			addMessage("Error: " + message);
		}

		public void onError(int num, String message) {
			addMessage("Error: " + message);
		}

		public void onInvite(String channel, IRCUser user, String invitedNick) {
			addMessage(user.getNick() + " has invited " + invitedNick + " to " + channel);
		}

		public void onJoin(String channel, IRCUser user) {
			addMessage(user.getNick() + " has joined " + channel);
		}

		public void onKick(String channel, IRCUser user, String kickedNick, String message) {
			addMessage(user.getNick() + " has kicked " + kickedNick);
		}

		public void onMode(String channel, IRCUser user, IRCModeParser modeParser) {
			addMessage(user.getNick() + " has set mode " + modeParser.getLine());
		}

		public void onMode(IRCUser user, String targetNick, String mode) {
			addMessage(user.getNick() + " has changed the mode of " + targetNick + " to " + mode);
		}

		public void onNick(IRCUser user, String newNick) {
			addMessage(user.getNick() + " is now known as " + newNick);
		}

		public void onNotice(String target, IRCUser user, String message) {
			addMessage("(" + user.getNick() + ") " + message);
		}

		public void onPart(String channel, IRCUser user, String message) {
			addMessage(user.getNick() + " has left channel (" + message + ")");
		}

		public void onPing(String ping) {
		}

		public void onPrivmsg(String target, IRCUser user, String message) {
			addMessage(user.getNick(), message);
		}

		public void onQuit(IRCUser user, String message) {
			addMessage(user.getNick() + " has left IRC (" + message + ")");
		}

		public void onRegistered() {
			addMessage("Connected");
			updateSayButton();
			conn.doJoin(channel);
		}

		public void onReply(int num, String value, String message) {
			addMessage(Integer.toString(num) + " " + message);
		}

		public void onTopic(String channel, IRCUser user, String topic) {
			addMessage(user.getNick() + " has changed the topic to " + topic);
		}

		public void unknown(String prefix, String command, String middle, String trailing) {
			addMessage(command + " " + middle + " :" + trailing);
		}
    	
    }

}